@@ -6,14 +6,17 @@
  */
 
 #include "data-types.h"
+#include "cleanup.h"
 #include <dlfcn.h>
 
 #define FUNC(name, restype, ...) typedef restype (*name##_func)(__VA_ARGS__); static name##_func name = NULL
 #define LOAD_FUNC(handle, name) {\
     *(void **) (&name) = dlsym(handle, #name); \
-    const char* error = dlerror(); \
-    if (error != NULL) { \
-        PyErr_Format(PyExc_OSError, "Failed to load the function %s with error: %s", #name, error); dlclose(handle); handle = NULL; return NULL; \
+    if (!name) { \
+        const char* error = dlerror(); \
+        if (error != NULL) { \
+            PyErr_Format(PyExc_OSError, "Failed to load the function %s with error: %s", #name, error); dlclose(handle); handle = NULL; return NULL; \
+        } \
     } \
 }
 
@@ -34,10 +37,14 @@ init_x11_startup_notification(PyObject UNUSED *self, PyObject *args) {
         done = true;
 
         const char* libnames[] = {
+#if defined(_KITTY_STARTUP_NOTIFICATION_LIBRARY)
+            _KITTY_STARTUP_NOTIFICATION_LIBRARY,
+#else
             "libstartup-notification-1.so",
             // some installs are missing the .so symlink, so try the full name
             "libstartup-notification-1.so.0",
             "libstartup-notification-1.so.0.0.0",
+#endif
             NULL
         };
         for (int i = 0; libnames[i]; i++) {
@@ -113,10 +120,14 @@ load_libcanberra(void) {
     if (done) return;
     done = true;
     const char* libnames[] = {
+#if defined(_KITTY_CANBERRA_LIBRARY)
+        _KITTY_CANBERRA_LIBRARY,
+#else
         "libcanberra.so",
         // some installs are missing the .so symlink, so try the full name
         "libcanberra.so.0",
         "libcanberra.so.0.2.5",
+#endif
         NULL
     };
     for (int i = 0; libnames[i]; i++) {
@@ -131,6 +142,7 @@ load_libcanberra(void) {
     if (PyErr_Occurred()) {
         PyErr_Print();
         dlclose(libcanberra_handle); libcanberra_handle = NULL;
+        return;
     }
     if (ca_context_create(&canberra_ctx) != 0) {
         fprintf(stderr, "Failed to create libcanberra context, cannot play beep sound\n");
@@ -163,9 +175,6 @@ finalize(void) {
 bool
 init_desktop(PyObject *m) {
     if (PyModule_AddFunctions(m, module_methods) != 0) return false;
-    if (Py_AtExit(finalize) != 0) {
-        PyErr_SetString(PyExc_RuntimeError, "Failed to register the desktop.c at exit handler");
-        return false;
-    }
+    register_at_exit_cleanup_func(DESKTOP_CLEANUP_FUNC, finalize);
     return true;
 }
