@@ -10,7 +10,9 @@ from .child import Child
 from .cli import WATCHER_DEFINITION, parse_args
 from .cli_stub import LaunchCLIOptions
 from .constants import resolve_custom_file
-from .fast_data_types import patch_color_profiles, set_clipboard_string
+from .fast_data_types import (
+    get_options, patch_color_profiles, set_clipboard_string
+)
 from .tabs import Tab
 from .types import run_once
 from .utils import find_exe, read_shell_environment, set_primary_selection
@@ -219,7 +221,7 @@ def load_watch_modules(watchers: Sequence[str]) -> Optional[Watchers]:
     ans = Watchers()
     for path in watchers:
         path = resolve_custom_file(path)
-        m = runpy.run_path(path, run_name='__kitty_watcher__')  # type: ignore
+        m = runpy.run_path(path, run_name='__kitty_watcher__')
         w = m.get('on_close')
         if callable(w):
             ans.on_close.append(w)
@@ -285,6 +287,20 @@ def launch(
         kw['location'] = opts.location
     if opts.copy_colors and active:
         kw['copy_colors_from'] = active
+    pipe_data: Dict[str, Any] = {}
+    if opts.stdin_source != 'none':
+        q = str(opts.stdin_source)
+        if opts.stdin_add_formatting:
+            if q in ('@screen', '@screen_scrollback', '@alternate', '@alternate_scrollback'):
+                q = '@ansi_' + q[1:]
+        if opts.stdin_add_line_wrap_markers:
+            q += '_wrap'
+        penv, stdin = boss.process_stdin_source(window=active, stdin=q, copy_pipe_data=pipe_data)
+        if stdin:
+            kw['stdin'] = stdin
+            if penv:
+                env.update(penv)
+
     cmd = args or None
     if opts.copy_cmdline and active_child:
         cmd = active_child.foreground_cmdline
@@ -298,10 +314,25 @@ def launch(
                         x = s
                 elif x == '@active-kitty-window-id':
                     x = str(active.id)
+                elif x == '@input-line-number':
+                    if 'input_line_number' in pipe_data:
+                        x = str(pipe_data['input_line_number'])
+                elif x == '@line-count':
+                    if 'lines' in pipe_data:
+                        x = str(pipe_data['lines'])
+                elif x in ('@cursor-x', '@cursor-y', '@scrolled-by'):
+                    if active is not None:
+                        screen = active.screen
+                        if x == '@scrolled-by':
+                            x = str(screen.scrolled_by)
+                        elif x == '@cursor-x':
+                            x = str(screen.cursor.x + 1)
+                        elif x == '@cursor-y':
+                            x = str(screen.cursor.y + 1)
             final_cmd.append(x)
         exe = find_exe(final_cmd[0])
         if not exe:
-            env = read_shell_environment(boss.opts)
+            env = read_shell_environment(get_options())
             if 'PATH' in env:
                 import shutil
                 exe = shutil.which(final_cmd[0], path=env['PATH'])
@@ -310,19 +341,6 @@ def launch(
         kw['cmd'] = final_cmd
     if opts.type == 'overlay' and active:
         kw['overlay_for'] = active.id
-    if opts.stdin_source != 'none':
-        q = str(opts.stdin_source)
-        if opts.stdin_add_formatting:
-            if q in ('@screen', '@screen_scrollback', '@alternate', '@alternate_scrollback'):
-                q = '@ansi_' + q[1:]
-        if opts.stdin_add_line_wrap_markers:
-            q += '_wrap'
-        penv, stdin = boss.process_stdin_source(window=active, stdin=q)
-        if stdin:
-            kw['stdin'] = stdin
-            if penv:
-                env.update(penv)
-
     if opts.type == 'background':
         cmd = kw['cmd']
         if not cmd:
