@@ -16,7 +16,7 @@ from .cli import (
     title
 )
 from .cli_stub import RCOptions
-from .constants import cache_dir, is_macos, version, kitty_face
+from .constants import cache_dir, version, kitty_face
 from .rc.base import (
     RemoteCommand, all_command_names, command_for_name,
     display_subcommand_help, parse_subcommand_cli
@@ -31,11 +31,8 @@ def match_commands() -> Tuple[str, ...]:
 
 
 def init_readline(readline: Any) -> None:
-    try:
+    with suppress(OSError):
         readline.read_init_file()
-    except OSError:
-        if not is_macos:
-            raise
     if 'libedit' in readline.__doc__:
         readline.parse_and_bind("bind ^I rl_complete")
     else:
