@@ -1091,6 +1091,8 @@ finalize(void) {
     free_bgimage(&global_state.bgimage, false);
     global_state.bgimage = NULL;
     free_url_prefixes();
+    free(OPT(select_by_word_characters)); OPT(select_by_word_characters) = NULL;
+    free(OPT(url_excluded_characters)); OPT(url_excluded_characters) = NULL;
 }
 
 bool
