@@ -14,19 +14,23 @@ from contextlib import suppress
 from functools import lru_cache
 from time import monotonic
 from typing import (
-    Any, Callable, Dict, Generator, Iterable, List, Mapping, Match, NamedTuple,
-    Optional, Tuple, Union, cast
+    TYPE_CHECKING, Any, Callable, Dict, Generator, Iterable, List, Mapping,
+    Match, NamedTuple, Optional, Tuple, Union, cast
 )
 
 from .constants import (
     appname, is_macos, is_wayland, read_kitty_resource, shell_path,
     supports_primary_selection
 )
-from .options_stub import Options
 from .rgb import Color, to_color
-from .types import ConvertibleToNumbers, run_once
+from .types import run_once
 from .typing import AddressFamily, PopenType, Socket, StartupCtx
 
+if TYPE_CHECKING:
+    from .options.types import Options
+else:
+    Options = object
+
 
 def expandvars(val: str, env: Mapping[str, str] = {}, fallback_to_os_env: bool = True) -> str:
 
@@ -72,9 +76,10 @@ def safe_print(*a: Any, **k: Any) -> None:
 
 def log_error(*a: Any, **k: str) -> None:
     from .fast_data_types import log_error_string
+    output = getattr(log_error, 'redirect', log_error_string)
     with suppress(Exception):
-        msg = k.get('sep', ' ').join(map(str, a)) + k.get('end', '')
-        log_error_string(msg.replace('\0', ''))
+        msg = k.get('sep', ' ').join(map(str, a)) + k.get('end', '').replace('\0', '')
+        output(msg)
 
 
 def ceil_int(x: float) -> int:
@@ -264,7 +269,14 @@ def init_startup_notification(window_handle: Optional[int], startup_id: Optional
         log_error('Could not perform startup notification as window handle not present')
         return None
     try:
-        return init_startup_notification_x11(window_handle, startup_id)
+        try:
+            return init_startup_notification_x11(window_handle, startup_id)
+        except OSError as e:
+            if not str(e).startswith("Failed to load libstartup-notification"):
+                raise e
+            log_error(
+                f'{e}. This has two main effects:',
+                'There will be no startup feedback and when using --single-instance, kitty windows may start on an incorrect desktop/workspace.')
     except Exception:
         import traceback
         traceback.print_exc()
@@ -514,8 +526,8 @@ def get_editor_from_env_vars(opts: Optional[Options] = None) -> List[str]:
 
 def get_editor(opts: Optional[Options] = None) -> List[str]:
     if opts is None:
-        from .cli import create_default_opts
-        opts = create_default_opts()
+        from .fast_data_types import get_options
+        opts = get_options()
     if opts.editor == '.':
         return get_editor_from_env_vars()
     import shlex
@@ -680,11 +692,3 @@ class SSHConnectionData(NamedTuple):
     binary: str
     hostname: str
     port: Optional[int] = None
-
-
-def positive_int(x: ConvertibleToNumbers) -> int:
-    return max(0, int(x))
-
-
-def positive_float(x: ConvertibleToNumbers) -> float:
-    return max(0, float(x))
